/***************************************************************************
                          kaddprinterwizard.cpp  -  description
                             -------------------
    begin                : Mon Jan 31 2000
    copyright            : (C) 2000 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qlayout.h>
#include <qfont.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include <qmessagebox.h>
#include <qwidgetstack.h>
#include <klocale.h>

//#include "kups.h"
#include "kaddprinterwizard.h"
#include <qstring.h>

// Wizard pages
#include "kap_explaination.h"
#include "kap_backendselector.h"
//#include "kap_parallelchooserpage.h"
#include "kap_modelselectorpage.h"
#include "kap_lastconfirmationpage.h"
#include "kap_namelocationselectorpage.h"
#include "kap_classselectorpage.h"
#include "kap_ippselector.h"
#include "kap_loginselector.h"
#include "kap_smbselector.h"
//#include "kap_parallelselector.h"
#include "kap_lpdinfopage.h"
#include "kap_lpdprinterselector.h"
#include "kap_socketselector.h"
#include "kap_ippprinterselector.h"
#include "kap_localselector.h"
#include "kap_fileselector.h"
#include "kap_driverconfig.h"
#include "kap_driverselectorpage.h"
#include "kap_driverdetectpage.h"
#include "kap_bannerselectorpage.h"


//---------------------------------------------------------------------------------------
Kap_AddPrinterWizard::Kap_AddPrinterWizard(QWidget *parent, const char * )
:QDialog(parent, "Add printer wizard", true),
currentPage_(0),
startPage_(Kap_Main::Explaination),
stopPage_(Kap_Main::Confirmation),
inclusive_(true)
{
	setCaption(i18n("Add printer wizard"));

	widgetStack_ = new QWidgetStack(this);
	int	ID(0);

	// First page
	explPage_ = new Kap_Explaination(widgetStack_);
	widgetStack_->addWidget(explPage_,ID++);
	// Backend page
	backendPage_ = new Kap_BackendSelector(widgetStack_);
	widgetStack_->addWidget(backendPage_,ID++);
	// IPP pages
	ippSelectorPage_ = new Kap_IppSelector(widgetStack_);
	widgetStack_->addWidget(ippSelectorPage_,ID++);
	// IPP printer selector
	ippPrinterSelectorPage_ = new Kap_IppPrinterSelector(widgetStack_);
	widgetStack_->addWidget(ippPrinterSelectorPage_,ID++);
	// Login selector
	loginSelectorPage_ = new Kap_LoginSelector(widgetStack_);
	widgetStack_->addWidget(loginSelectorPage_,ID++);
	// SMB selector
	smbSelectorPage_ = new Kap_SmbSelector(widgetStack_);
	widgetStack_->addWidget(smbSelectorPage_,ID++);
	// LPD selector
	lpdSelectorPage_ = new Kap_LpdInfoPage(widgetStack_);
	widgetStack_->addWidget(lpdSelectorPage_,ID++);
	// LPD printer selector
	lpdPrinterSelectorPage_ = new Kap_LpdPrinterSelector(widgetStack_);
	widgetStack_->addWidget(lpdPrinterSelectorPage_,ID++);
	// Socket selector
	socketSelectorPage_ = new Kap_SocketSelector(widgetStack_);
	widgetStack_->addWidget(socketSelectorPage_,ID++);
	// Model selector
	modelSelectorPage_ = new Kap_ModelSelectorPage(widgetStack_);
	widgetStack_->addWidget(modelSelectorPage_,ID++);
	// Driver selector
	driverSelectorPage_ = new Kap_DriverSelectorPage(widgetStack_);
	widgetStack_->addWidget(driverSelectorPage_,ID++);
	// Driver config
	driverConfigPage_ = new Kap_DriverConfigPage(widgetStack_);
	widgetStack_->addWidget(driverConfigPage_,ID++);
	// Banner selection
	bannerSelectorPage_ = new Kap_BannerSelectorPage(widgetStack_);
	widgetStack_->addWidget(bannerSelectorPage_,ID++);
	// Name and location selector
	nameSelectorPage_ = new Kap_NameLocationSelectorPage(widgetStack_);
	widgetStack_->addWidget(nameSelectorPage_,ID++);
	// Class selector
	classSelectorPage_ = new Kap_ClassSelectorPage(widgetStack_);
	widgetStack_->addWidget(classSelectorPage_,ID++);
	// Local printer selector
	localSelectorPage_ = new Kap_LocalSelector(widgetStack_);
	widgetStack_->addWidget(localSelectorPage_,ID++);
	// Local driver selection
	driverDetectPage_ = new Kap_DriverDetectPage(widgetStack_);
	widgetStack_->addWidget(driverDetectPage_,ID++);
	// File selector
	fileSelectorPage_ = new Kap_FileSelector(widgetStack_);
	widgetStack_->addWidget(fileSelectorPage_,ID++);
	// Last page
	confirmPage_ = new Kap_LastConfirmationPage(widgetStack_);
	widgetStack_->addWidget(confirmPage_,ID++);

  QFont f("Helvetica", 16, QFont::Bold);
	pageTitle_ = new QLabel(this);
	pageTitle_->setFont(f);
	cancelButton_ = new QPushButton(i18n("Cancel"), this, "CancelButton");
	cancelButton_->setFixedSize(cancelButton_->sizeHint());
	prevButton_ = new QPushButton(i18n("< Back"), this, "PrevButton");
	prevButton_->setFixedSize(prevButton_->sizeHint());
	nextButton_ = new QPushButton(i18n("Next >"), this, "NextButton");
	nextButton_->setFixedSize(nextButton_->sizeHint());
	nextButton_->setDefault(true);

	initLayout();

	setCurrentPage(startPage_, false);

	setMinimumSize(300, 200);
	resize(400, 300);

	makeConnection();

	initialize();
}
Kap_AddPrinterWizard::~Kap_AddPrinterWizard()
{
}

void Kap_AddPrinterWizard::initLayout()
{
	QGridLayout* lowerLayout = new QGridLayout(1, 7);

	QGridLayout* mainLayout = new QGridLayout(this, 7, 3);
	mainLayout->addWidget(pageTitle_, 1, 1);
	mainLayout->addWidget(widgetStack_, 3, 1);
	mainLayout->addLayout(lowerLayout, 5, 1);
	
	mainLayout->addColSpacing(0, 5);
	mainLayout->addColSpacing(1, 5);
	mainLayout->addColSpacing(2, 5);
	mainLayout->setColStretch(0, 1);
	mainLayout->setColStretch(1, 20);
	mainLayout->setColStretch(2, 1);
	
	mainLayout->addRowSpacing(0, 5);
	mainLayout->addRowSpacing(2, 5);
	mainLayout->addRowSpacing(4, 5);
	mainLayout->addRowSpacing(6, 5);
	mainLayout->setRowStretch(0, 0);
	mainLayout->setRowStretch(1, 1);
	mainLayout->setRowStretch(2, 0);
	mainLayout->setRowStretch(3, 10);
	mainLayout->setRowStretch(4, 0);
	mainLayout->setRowStretch(5, 1);
	mainLayout->setRowStretch(6, 0);
	
	
	
	lowerLayout->addColSpacing(0, 5);
	lowerLayout->addWidget(cancelButton_, 0, 1);
	lowerLayout->addColSpacing(2, 50);
	lowerLayout->addWidget(prevButton_, 0, 3);
	lowerLayout->addColSpacing(4, 5);
	lowerLayout->addWidget(nextButton_, 0, 5);
	lowerLayout->addColSpacing(6, 5);

	mainLayout->activate();
}

void Kap_AddPrinterWizard::initWidgets()
{
	pageTitle_->setText( currentPage_->getTitle() );
	pageTitle_->setMinimumSize( pageTitle_->sizeHint() );
	//pageTitle_->setMaximumSize( pageTitle_->sizeHint() );

	QString	dummy;
	if (currentPage_->getType() == stopPage_ || (currentPage_->getNextPage(dummy, true) == stopPage_ && !inclusive_))
	{
		nextButton_->setText(i18n("OK"));
		nextButton_->show();
	}
	else if (currentPage_->hasNextPage())
	{
		nextButton_->setText(i18n("Next >"));
		nextButton_->show();
	}
	else
	{
		nextButton_->hide();
	}
	if (currentPage_->getType() == startPage_)
	{
		prevButton_->hide();
	}
	else
	{
		prevButton_->show();
	}

/*	if (currentPage_->hasNextPage())
	{
		nextButton_->setText(i18n("Next >"));
		nextButton_->show();
	}
	else if (currentPage_->hasOkButton())
	{
		nextButton_->setText(i18n("OK"));
		nextButton_->show();
	}
	else
	{
		nextButton_->hide();
	}
	if (currentPage_->hasPrevPage())
	{
		prevButton_->show();
	}
	else
	{
		prevButton_->hide();
	}*/
	widgetStack_->raiseWidget(currentPage_);
}

void Kap_AddPrinterWizard::makeConnection()
{
	connect(cancelButton_, SIGNAL(clicked()), this, SLOT(slotCancel()) );
	connect(nextButton_, SIGNAL(clicked()), this, SLOT(slotNext()) );
	connect(prevButton_, SIGNAL(clicked()), this, SLOT(slotPrev()) );
}

bool Kap_AddPrinterWizard::initialize()
{
	return (true);
}

void Kap_AddPrinterWizard::slotCancel()
{
	bool ret = currentPage_->slotCancel();
	done(ret);
	return;
}

void Kap_AddPrinterWizard::slotNext()
{
	QString	msg;
	if (!currentPage_->canNextPage(msg))
	{
		if (!msg.isEmpty()) QMessageBox::information(this,i18n("Continue request"),msg.data(),QMessageBox::Ok|QMessageBox::Default,0);
		return;
	}
	msg = "";
	Kap_Main::e_PageType	type = currentPage_->getNextPage(msg);
	// simple hack to prevent the driverDetectPage_ to appear when modifying a printer
	if (type == Kap_Main::DriverDetectType && startPage_ != Kap_Main::Explaination)
		type = Kap_Main::ModelSelector;

	if ( i18n("OK") == nextButton_->text()  && ((inclusive_ && currentPage_->getType() == stopPage_) || (!inclusive_ && type == stopPage_)))
	{
		bool ret = currentPage_->slotOk();
		done(ret);
		return;
	}
	else
	{
		if (type == Kap_Main::EndOfPages)
		{
			done(1);
			return;
		}
		else if (type == Kap_Main::Invalid && currentPage_->getType() != stopPage_)
		{
			QMessageBox::critical(this,i18n("Add printer wizard"),i18n("Can't find the page specified ! ('%1' not yet implemented ?)").arg((int)type),QMessageBox::Ok|QMessageBox::Default,0);
			done(0);
			return;
		}
		else
		{
			setCurrentPage(type, true);
		}
	}
}

void Kap_AddPrinterWizard::slotPrev()
{
	Kap_Main::e_PageType type = getPrevPage();
	setCurrentPage(type, false);
}

Kap_WizardPage* Kap_AddPrinterWizard::getPage(Kap_Main::e_PageType type)
{
	switch(type)
	{
	case Kap_Main::Explaination:
		return explPage_;

	case Kap_Main::BackendSelector:
		return backendPage_;

	case Kap_Main::ClassSelector:
		return classSelectorPage_;

	case Kap_Main::IppSelector:
		return ippSelectorPage_;
		
	case Kap_Main::LoginSelector:
		return loginSelectorPage_;
		
	case Kap_Main::SmbSelector:
		return smbSelectorPage_;

	case Kap_Main::ModelSelector:
		return modelSelectorPage_;
	
	case Kap_Main::Confirmation:
		return confirmPage_;
		
	case Kap_Main::NameSelector:
		return nameSelectorPage_;
		
/*	case Kap_Main::ParallelSelector:
		return parallelSelector_;*/
		
	case Kap_Main::LpdSelector:
		return lpdSelectorPage_;
		
	case Kap_Main::LpdPrinterSelector:
		return lpdPrinterSelectorPage_;

	case Kap_Main::SocketSelector:
		return socketSelectorPage_;

	case Kap_Main::IppPrinterSelector:
		return ippPrinterSelectorPage_;

	case Kap_Main::LocalSelector:
		return localSelectorPage_;

	case Kap_Main::FileSelector:
		return fileSelectorPage_;

	case Kap_Main::DriverConfig:
		return driverConfigPage_;

	case Kap_Main::DriverSelector:
		return driverSelectorPage_;

	case Kap_Main::DriverDetectType:
		return driverDetectPage_;

	case Kap_Main::BannerSelector:
		return bannerSelectorPage_;

	case Kap_Main::Invalid:
	default:
		break;
	};

	return 0;
}

Kap_Main::e_PageType Kap_AddPrinterWizard::getPrevPage()
{
	Kap_Main::e_PageType* type = prevPageList_.at(prevPageList_.count()-1);
	prevPageList_.remove(prevPageList_.count()-1);
	return *type;
}
void Kap_AddPrinterWizard::addPrevPage(Kap_Main::e_PageType type)
{
	prevPageList_.append( new Kap_Main::e_PageType(type) );
}

void Kap_AddPrinterWizard::setCurrentPage(Kap_Main::e_PageType type, bool addToPrev)
{
	Kap_WizardPage* page = getPage(type);
	if (page == 0)
	{
		QMessageBox::critical(this,i18n("Add printer wizard"),i18n("Can't get the page specified (not yet implemented ?)"),QMessageBox::Ok|QMessageBox::Default,0);
		return;
	}
	
	if (addToPrev)
	{
		Kap_Main::e_PageType type = currentPage_->getType();
		addPrevPage(type);
	}
	
	//currentPage_->hide();
	currentPage_ = page;
	
	initWidgets();
	
	currentPage_->initialize();
}

void Kap_AddPrinterWizard::setWizardControl(Kap_Main::e_PageType start, Kap_Main::e_PageType stop, bool incl)
{
	startPage_ = start;
	stopPage_ = stop;
	inclusive_ = incl;

	// restart from the specified page (normally called before wizard execution)
	prevPageList_.clear();
	setCurrentPage(startPage_, false);

	// if startPage_ isn't Explaination, then we disable "class"
	((Kap_BackendSelector*)backendPage_)->enableClass(startPage_ == Kap_Main::Explaination);

	// change caption if modifying
	if (startPage_ != Kap_Main::Explaination)
		setCaption(i18n("Modify printer wizard"));
}
