/***************************************************************************
                          kap_bannerselectorpage.cpp  -  description
                             -------------------
    begin                : Tue Feb 8 2000
    copyright            : (C) 2000 by Michael Goffioul
    email                : goffioul@@imec.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <klocale.h>

#include "kap_bannerselectorpage.h"
#include "kprinterdescription.h"
#include "kprintermanager.h"
#include "kprinter.h"

void findLocalAvailableBanners(QStringList& list)
{
	KPrinterManager	*mgr = KPrinterManager::get();
	for (int i=0;i<mgr->getNbPrinters();i++)
	{
		KPrinter	*pr = mgr->getPrinterAt(i);
		if (!pr->isRemote() && !pr->isImplicit() && !pr->isClass())
		{
			list = pr->getSupportedBanners();
			return;
		}
	}
	list.clear();
	list.append("none");
}

void initComboItem(QComboBox *cb, const QString& txt)
{
	for (int i=0;i<cb->count();i++)
		if (cb->text(i) == txt)
		{
			cb->setCurrentItem(i);
			break;
		}
}

//****************************************************************************************

Kap_BannerSelectorPage::Kap_BannerSelectorPage(QWidget *parent, const char *name )
:Kap_WizardPage(parent,name)
{
	setTitle(i18n("Banners selector"));

	QLabel	*explLabel_ = new QLabel(this);
	explLabel_->setText(i18n("<p>Select the banners you want to use for this printer, or use <b>none</b> if you don't want to use banners. Banner pages will be inserted before and/or after each print job sent to this printer.</p>"));

	QLabel	*startLabel_ = new QLabel(this);
	startLabel_->setText(i18n("Starting banner:"));

	QLabel	*endLabel_ = new QLabel(this);
	endLabel_->setText(i18n("Ending banner:"));

	startbanner_ = new QComboBox(this);
	endbanner_ = new QComboBox(this);

	QVBoxLayout	*left_ = new QVBoxLayout(0, 0, 10), *right_ = new QVBoxLayout(0, 0, 10);
	QHBoxLayout *sub_ = new QHBoxLayout(0, 0, 20);
	QVBoxLayout	*main_ = new QVBoxLayout(this, 10, 30);
	main_->addWidget(explLabel_);
	main_->addLayout(sub_);
	main_->addStretch(1);
	sub_->addLayout(left_);
	sub_->addLayout(right_);
	left_->addWidget(startLabel_);
	left_->addWidget(endLabel_);
	right_->addWidget(startbanner_);
	right_->addWidget(endbanner_);
}

Kap_BannerSelectorPage::~Kap_BannerSelectorPage()
{
}

bool Kap_BannerSelectorPage::initialize()
{
        KPrinterDescription	*descr = KPrinterDescription::get();
	QStringList	banners = descr->getSupportedBanners();

	// probably only have "none", try to find available banners from another local printer
	if (banners.count() <=1) findLocalAvailableBanners(banners);

	startbanner_->clear();
	startbanner_->insertStringList(banners);
	endbanner_->clear();
	endbanner_->insertStringList(banners);

	initComboItem(startbanner_,(descr->getStartBanner().isEmpty() ? QString("none") : descr->getStartBanner()));
	initComboItem(endbanner_,(descr->getEndBanner().isEmpty() ? QString("none") : descr->getEndBanner()));

	return true;
}

Kap_Main::e_PageType Kap_BannerSelectorPage::getNextPage(QString&, bool fake)
{
	if (!fake)
	{
		KPrinterDescription* parDet = KPrinterDescription::get();
		parDet->setStartBanner(startbanner_->currentText());
		parDet->setEndBanner(endbanner_->currentText());
	}
	return Kap_Main::NameSelector;
}
Kap_Main::e_PageType Kap_BannerSelectorPage::getType() const
{
	return Kap_Main::BannerSelector;
}
