

#include "kap_dbcreator.h"

#include <qprogressbar.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qfileinfo.h>
#include <qdir.h>

#include <kapp.h>
#include <klocale.h>
#include <kglobal.h>
#include <kstddirs.h>

Kap_DBCreator::Kap_DBCreator(QWidget *parent, const char *name)
	: QDialog(parent,name,true)
{
	setCaption(i18n("Database builder"));

	text_ = new QLabel(i18n("Please wait while kups is rebuilding driver database"),this);
	cancel_ = new QPushButton(i18n("Cancel"),this);
	bar_ = new QProgressBar(this);
	first_ = false;
	ppdDirname_ = "/usr/share/cups/model";
	dbFilename_ = KGlobal::dirs()->localkdedir() + "/share/apps/kups/printers_db.txt";
	//dbFilename_ = KApplication::localkdedir() + "/share/apps/kups/printers_db.txt";

	connect(cancel_,SIGNAL(clicked()),SLOT(cancelClicked()));
	connect(&process_,SIGNAL(receivedStdout(KProcess*,char*,int)),SLOT(receivedStdout(KProcess*,char*,int)));
	connect(&process_,SIGNAL(receivedStderr(KProcess*,char*,int)),SLOT(receivedStderr(KProcess*,char*,int)));
	connect(&process_,SIGNAL(processExited(KProcess*)),SLOT(processExited(KProcess*)));

	int	w;
	w = text_->sizeHint().width()+20;
	resize(w,120);
}

Kap_DBCreator::~Kap_DBCreator()
{
	if (process_.isRunning()) process_.kill();
}

void Kap_DBCreator::setPpdDirectory(const QString& dirname)
{
	ppdDirname_ = dirname;
}
void Kap_DBCreator::setDbFilename(const QString& dirname)
{
	dbFilename_ = dirname;
	QFileInfo info(dbFilename_);
	QDir dir = info.dir(true);
	QDir root("/");
	if ( !KGlobal::dirs()->makeDir(dir.absPath()) )
	{
#ifdef DEBUG
debug("Can't make drivers DB directory\n");
#endif
		return;
	}
}

void Kap_DBCreator::show()
{
	startProcess();
	QDialog::show();
}

void Kap_DBCreator::resizeEvent(QResizeEvent*)
{
	QSize	st(text_->sizeHint()),sc(cancel_->sizeHint()),sb(bar_->sizeHint());
	int	s((height()-st.height()-sc.height()-sb.height())/4);
	text_->setGeometry(10,s,width()-20,st.height());
	bar_->setGeometry(10,2*s+st.height(),width()-20,sb.height());
	cancel_->setGeometry(width()-10-sc.width(),3*s+st.height()+sb.height(),sc.width(),sc.height());
}

void Kap_DBCreator::startProcess()
{
	if (process_.isRunning()) process_.kill();
	process_.clearArguments();
	errorString_ = "";
	first_ = true;

	QString	cmd("make_drivers_db");
	//QString	cmd(ppdDirname_+"/make_drivers_db");
	//QString cmd(dbdirname_+"/make_drivers_db.pl");
	
	//QString	outputDir(KApplication::localkdedir()+"/share/apps/kups");
	//QString	outputDir("/usr/share/cups/model");

#ifdef DEBUG
debug("Dirs |%s|%s|\n", ppdDirname_.data(), dbFilename_.data() );
#endif
	
	process_.setExecutable(cmd.data());
	process_ << ppdDirname_.data();
	process_ << dbFilename_.data();
	process_.start(KProcess::NotifyOnExit, KProcess::AllOutput);
#ifdef DEBUG
debug("Process started\n");
#endif
}

void Kap_DBCreator::receivedStdout(KProcess*, char *buffer, int len)
{
	QString	str( QCString(buffer, len) );
	str.stripWhiteSpace();
	bool	ok;
	int	num = str.toInt(&ok);
	if (ok)
	{
#ifdef DEBUG
debug("OK : %d", num);
#endif
		if (first_)
		{
			bar_->setTotalSteps(num);
			first_ = false;
		}
		else
		{	
			bar_->setProgress(num);
		}
	}
#ifdef DEBUG
	else
	{
debug("NOK : |%s|", str.data() );
	}
#endif
	/*
	else
	{
		cancelClicked();
	}
	*/
}

void Kap_DBCreator::receivedStderr(KProcess*, char *buffer, int len)
{
	QString	str( QCString(buffer,len) );
#ifdef DEBUG
debug("error: %s",str.data());
#endif
	errorString_ = str.data();
}

void Kap_DBCreator::processExited(KProcess*)
{
	if (!process_.normalExit())
	{
		errorString_ = "Process was killed";
		reject();
	}
	else if (process_.exitStatus() != 0) reject();
	else accept();
}

void Kap_DBCreator::cancelClicked()
{
	if (process_.isRunning()) process_.kill();
	errorString_ = "DB creation cancelled";
	reject();
}

QString& Kap_DBCreator::errorString()
{
	return errorString_;
}
