#include "kap_driverselector.h"

#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qmessagebox.h>

#include <klocale.h>

#include <cups/ppd.h>
#include <qtcups/globalppd.h>

#include "kap_manumodeldico.h"
#include "kap_printerconfig.h"

Kap_DriverSelector::Kap_DriverSelector(QWidget *parent, const char *name)
	: QDialog(parent,name,true)
{
	QLabel	*lab_ = new QLabel(i18n("More than one driver detected. Select one:"), this);
	lab_->setFixedHeight(lab_->sizeHint().height());
	listview_ = new QListView(this);
	listview_->addColumn(i18n("Manufacturer"));
	listview_->addColumn(i18n("Model"));
	listview_->addColumn(i18n("Driver"));
	listview_->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	listview_->setLineWidth(1);
	listview_->setAllColumnsShowFocus(true);
	listview_->setSorting(-1);
	QPushButton	*ok_ = new QPushButton(i18n("OK"), this);
	ok_->setFixedSize(ok_->sizeHint());
	ok_->setDefault(true);
	QPushButton	*cancel_ = new QPushButton(i18n("Cancel"), this);
	cancel_->setFixedSize(cancel_->sizeHint());
	connect(ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(cancel_, SIGNAL(clicked()), SLOT(reject()));
	options_ = new QPushButton(i18n("Settings..."), this);
	options_->setFixedSize(options_->sizeHint());
	connect(options_, SIGNAL(clicked()), SLOT(optionsClicked()));

	QVBoxLayout	*main_ = new QVBoxLayout(this, 10, 10);
	main_->addWidget(lab_);
	main_->addWidget(listview_, 1);
	QHBoxLayout	*sub_ = new QHBoxLayout(0, 0, 10);
	main_->addLayout(sub_);
	sub_->addWidget(options_);
	sub_->addStretch(1);
	sub_->addWidget(ok_);
	sub_->addWidget(cancel_);

	list_ = 0;
	dict_.setAutoDelete(false);
	initView();

	setCaption(i18n("Select driver"));
	resize(350, 200);
}

Kap_DriverSelector::~Kap_DriverSelector()
{
}

void Kap_DriverSelector::initView()
{
	listview_->clear();
	dict_.clear();
	if (!list_ || list_->count() == 0) return;
	for (list_->first();list_->current();list_->next())
	{
		QString	ppdName = list_->current()->getPpdFile();
		int	pos = ppdName.findRev('/');
		if (pos != -1) ppdName = ppdName.right(ppdName.length()-pos-1);
		dict_.insert(ppdName, list_->current());
		new QListViewItem(listview_, list_->current()->getManufacturer(), list_->current()->getNickName(), ppdName);
	}
	listview_->setSelected(listview_->firstChild(), true);
}

void Kap_DriverSelector::setModelList(QList<Kap_ModelItem> *list)
{
	list_ = list;
	initView();
}

Kap_ModelItem* Kap_DriverSelector::selected()
{
	if (!list_ || !listview_->currentItem()) return 0;
	return dict_.find(listview_->currentItem()->text(2));
}

void Kap_DriverSelector::optionsClicked()
{
	Kap_ModelItem	*item = selected();
	if (item && !item->getPpdFile().isEmpty())
	{
		global_ppd_file_t	*ppd = globalPpdOpenFile(item->getPpdFile().data());
		if (ppd)
		{
			globalPpdMarkDefaults(ppd);
			Kap_PrinterConfig	dlg(ppd, this);
			dlg.setUseConflict(false);
			dlg.exec();
			globalPpdClose(ppd);
		}
		else
		{
			QMessageBox::critical(this, i18n("Driver error"), i18n("Unable to find driver file \"%1\" !").arg(item->getPpdFile()), QMessageBox::Ok|QMessageBox::Default, 0);
		}
	}
}

Kap_ModelItem* Kap_DriverSelector::selectDriver(QList<Kap_ModelItem> *list, QWidget *parent, bool enableOptions)
{
	Kap_DriverSelector	dlg(parent);
	dlg.setModelList(list);
        dlg.setOptionsEnabled(enableOptions);
	if (dlg.exec())
	{
		return dlg.selected();
	}
	else return 0;
}

void Kap_DriverSelector::setOptionsEnabled(bool on)
{
        if (on) options_->show();
        else options_->hide();
}
