
#include "kups.h"
#include "kap_localselector.h"
#include "kap_ipputil.h"
#include "kprinterdescription.h"
#include "kap_manumodeldico.h"
#include "kbuffer.h"
#include "kap_driverselector.h"

#include <qlistview.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qheader.h>
#include <qmessagebox.h>

#include <kapp.h>
#include <kiconloader.h>

int getPortNumber(const QString& str)
{ // extract the port number in string in the form "xxx #<number>"
	int	p = str.findRev('#');
	if (p == -1) return -1;
	else
	{
		bool	ok;
		int	port = str.right(str.length()-p-1).toInt(&ok);
		return (ok ? port : -1);
	}
}

//------------------------------------------------------------------------------------

Kap_LocalSelector::Kap_LocalSelector(QWidget *parent, const char *name)
	: Kap_WizardPage(parent,name)
{
	setTitle(i18n("Local printer port"));

	list_ = new QListView(this);
	list_->addColumn("");
	list_->header()->hide();
	list_->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	connect(list_,SIGNAL(selectionChanged(QListViewItem*)),SLOT(listClicked(QListViewItem*)));

	speedLabel_ = new QLabel(i18n("Transfer speed"),this);

	speed_ = new QComboBox(this);
	speed_->insertItem("115200");
	speed_->insertItem("57600");
	speed_->insertItem("28800");
	speed_->insertItem("14400");

	initialized_ = false;
}

Kap_LocalSelector::~Kap_LocalSelector()
{
}

Kap_Main::e_PageType Kap_LocalSelector::getType() const
{
	return Kap_Main::LocalSelector;
}

Kap_Main::e_PageType Kap_LocalSelector::getNextPage(QString& msg, bool fake)
{
	if (fake) return Kap_Main::ModelSelector;

	QListViewItem* item = list_->currentItem();
	QListViewItem* parent = item->parent();
	KPrinterDescription	*descr = KPrinterDescription::get();

	// case of port selected, and detected printer on the port
	if (item->depth() == 2)
	{
		// This is an item selected => take its parent
		parent = parent->parent();
	}
	else if (item->childCount() > 0)
	{
		// Port is selected => Take the first item on this port
		item = item->firstChild();
	}

	// setting printer type
	KPrinterManager::e_AddType type(KPrinterManager::e_Invalid);
	if (parent == ports_[0])
	{
		type = KPrinterManager::e_Parallel;
	}
	else if (parent == ports_[1])
	{
		type = KPrinterManager::e_Serial;
	}
	else if (parent == ports_[2])
	{
		type = KPrinterManager::e_USB;
	}

	// setting URI
	descr->setURI(QString(item->text(1)));
	descr->setType(type);
	descr->setPrinterValues(0);
	descr->setBuffer(0);

	// setting printer infos (for detected printers)
	if (item->depth() == 2)
	{
		QString	make, model, str(item->text(0));
		// first parse printer string to retrieve manufacturer and model
		int	p(str.find(' '));
		if (p != -1)
		{
			make = str.left(p).stripWhiteSpace();
			model = str.right(str.length()-p-1).stripWhiteSpace();
			// update printer description
			descr->setModel(model);
			descr->setManufacturer(make);
			descr->setDescription(str);
			// look for driver
			Kap_ManuModelDico	*dico = Kap_ManuModelDico::get(true);
			if (dico == 0)
			{
				msg = i18n("Can't find driver database. Specify the driver location.");
				return Kap_Main::ModelSelector;
			}
			Kap_ModelItemList*	values = dico->getDescriptionForPnp(make,model);
			if (values == 0)
				return Kap_Main::ModelSelector;
			else
			{
				if (values->count() == 1) descr->setPrinterValues(values->first());
				return Kap_Main::DriverDetectType;
			}
/*			if (values != 0)
			{ // drivers found
				Kap_ModelItem	*value = values->first();
				if (values->count() == 1)
				{ // only 1 driver found, ask user if we use it.
					if (QMessageBox::information(this,i18n("Printer driver"),i18n("A driver \"%1\" has been found. Do you want to use it ?").arg(value->getNickName()),QMessageBox::Yes|QMessageBox::Default,QMessageBox::No) == QMessageBox::No)
						// do not use this driver, select one manually
						return Kap_Main::ModelSelector;
				}
				else
				{ // several drivers, ask user to select one.
					value = Kap_DriverSelector::selectDriver(values, this);
					if (!value) return Kap_Main::ModelSelector;
				}
				descr->setPrinterValues(value);
				KBuffer	*buffer = new KBuffer;
				if (dico->getValidPpdFileAsBuffer(*value,*buffer) != 0)
				{ // valid PPD file
					descr->setBuffer(buffer);
					return Kap_Main::NameSelector;
				}
				else
				{
					delete buffer;
				}
			}
			else
			{
				msg = i18n("Can't find appropriate driver in database.\n"
					   "Specify the driver location manually.");
				return Kap_Main::ModelSelector;
			}
*/
		}
	}
	// reset some values
	descr->setManufacturer("");
	descr->setModel("");
	descr->setDescription("");
	descr->setBuffer(0);
	descr->setPrinterValues(0);
	
	return Kap_Main::ModelSelector;
}

bool Kap_LocalSelector::canNextPage(QString& msg)
{
	QListViewItem	*item = list_->currentItem();
	if (!item || item->depth() == 0)
	{
		msg = i18n("You must select a port/printer !");
		return false;
	}
	if (item->depth() == 1 && item->childCount() <= 0)
	{
		if (QMessageBox::warning(this,i18n("Warning"),i18n("No printer detected on this port. Continue ?"),i18n("Yes"),i18n("No"),0,1) == 1)
		{
			msg = i18n("Select a port/printer !");
			return false;
		}
	}
	return true;
}

bool Kap_LocalSelector::initialize()
{
	if (initialized_) return true;
	initialized_ = true;
	speed_->setEnabled(false);
  list_->clear();

	ports_[0] = new QListViewItem(list_,i18n("Parallel ports"));
	ports_[0]->setSelectable(false);
	ports_[0]->setPixmap(0,SmallIcon("kups_port"));
	ports_[1] = new QListViewItem(list_,i18n("Serial ports"));
	ports_[1]->setSelectable(false);
	ports_[1]->setPixmap(0,SmallIcon("kups_port"));
	ports_[2] = new QListViewItem(list_,i18n("USB ports"));
	ports_[2]->setSelectable(false);
	ports_[2]->setPixmap(0,SmallIcon("kups_port"));

	QListViewItem* firstPrinter = 0;
	QList<DeviceInfo>	*devlist = Kap_IppUtil::getLocalPrinters();
	if (devlist)
	{
		QListIterator<DeviceInfo>	it(*devlist);
		bool	found(false);
		for (;it.current();++it)
		{
			QListViewItem	*parent(0), *item(0);
			if (it.current()->Type >= 0 && it.current()->Type < 3)
			{
				parent = ports_[it.current()->Type];
			}
			else
			{
				continue;
			}
			item = new QListViewItem(parent,it.current()->Info.data(),it.current()->URI.data());
			item->setPixmap(0,SmallIcon("kups_parport"));
			parent->setOpen(true);
			if (it.current()->MakeAndModel != "Unknown")
			{ // printer detected
				QListViewItem	*pitem = new QListViewItem(item,it.current()->MakeAndModel.data(),it.current()->URI.data());
				pitem->setPixmap(0,SmallIcon("kups_printer"));
				item->setOpen(true);
				if (!found)
				{
					firstPrinter = pitem;
				}
				found = found || true;
			}
		}

		if (!found)
		{
			QMessageBox::warning(this,i18n("Warning"),i18n("<p>No printer was detected on your system ! Maybe the printer was not "
								       "turned on when CUPS server started. Turn on your printer and restart "
								       "CUPS server, or select the port where the printer is connected.</p>"),QMessageBox::Ok|QMessageBox::Default,0);
		}
		
		if (firstPrinter)
		{
			list_->setSelected(firstPrinter, true);
		}
		
		delete devlist;
	}
	else
	{
		QMessageBox::warning(this,i18n("Warning"),i18n("No port detected on this computer !"),QMessageBox::Ok|QMessageBox::Default,0);
	}

	return true;
}

void Kap_LocalSelector::resizeEvent(QResizeEvent*)
{
	QSize	s(speed_->sizeHint());
	int	h(height()-20-s.height()-10), w((width()-10)/2);
	list_->setGeometry(5,15,width()-10,h);
	speedLabel_->setGeometry(5,h+25,w,s.height());
	speed_->setGeometry(5+w,h+25,w,s.height());
}

void Kap_LocalSelector::listClicked(QListViewItem *item)
{
	if (!item) return;
	QListViewItem	*parent(item->depth() == 2 ? item->parent()->parent() : item->parent());
	if (parent == ports_[1])
	{
		speed_->setEnabled(true);
	}
	else speed_->setEnabled(false);
}
