
#include "kap_loginselector.h"
#include "kprinterdescription.h"
#include "kups.h"

#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <kapp.h>

#include <stdlib.h>

Kap_LoginSelector::Kap_LoginSelector(QWidget *parent, const char *name)
	: Kap_WizardPage(parent,name)
{
	setTitle(i18n("User identification"));

	// create widgets
	infotext_ = new QLabel(this);
	infotext_->setText(i18n("<p>This backend may require a login/password to work properly. "
				"Fill in the required entries or enable <b>Guest account</b> to "
				"use anonymous access.</p>"));
	login_ = new QLineEdit(this);
	login_->setText(getenv("USER"));
	passwd_ = new QLineEdit(this);
	passwd_->setEchoMode(QLineEdit::Password);
	loginlabel_ = new QLabel(i18n("Login:"),this);
	passwdlabel_ = new QLabel(i18n("Password:"),this);
	guest_ = new QCheckBox(i18n("Guest account"),this);

	guest_->setChecked(true);
	login_->setEnabled(false);
	passwd_->setEnabled(false);
	connect(guest_,SIGNAL(clicked()),SLOT(guestClicked()));
}

Kap_LoginSelector::~Kap_LoginSelector()
{
}

void Kap_LoginSelector::resizeEvent(QResizeEvent *)
{
	QSize	s(infotext_->sizeHint());
	int	off(15);
	infotext_->setGeometry(5,off,width()-10,s.height());
	int	h((height()-5-off-s.height())/2);
	guest_->setGeometry(5,height()-5-h,width()-10,h);
	int	h1(s.height()+off+(h-2*25-5)/2), w((width()-10)/2);
	loginlabel_->setGeometry(5,h1,w,25);
	login_->setGeometry(5+w,h1,w,25);
	h1 += (25+5);
	passwdlabel_->setGeometry(5,h1,w,25);
	passwd_->setGeometry(5+w,h1,w,25);
}

Kap_Main::e_PageType Kap_LoginSelector::getType() const
{
	return Kap_Main::LoginSelector;
}

bool Kap_LoginSelector::canNextPage(QString& msg)
{
	if (!guest_->isChecked() && QString(passwd_->text()).isEmpty())
	{
		msg = i18n("Empty password, non-guest access requires one.");
		return false;
	}
	return true;
}

Kap_Main::e_PageType Kap_LoginSelector::getNextPage(QString&, bool fake)
{
	if (fake) return Kap_Main::SmbSelector;

	// suppose everything is correct (canNextPage is true).
	KPrinterDescription	*descr = KPrinterDescription::get();
	if (!guest_->isChecked())
	{
		descr->setLoginInfo(QString(login_->text()),QString(passwd_->text()));
	}
	else
	{
		descr->setLoginInfo(QString(""),QString(""));
	}
	
	switch (descr->getType())
	{
	case KPrinterManager::e_SMB:
		return Kap_Main::SmbSelector;
		
	case KPrinterManager::e_IPP:
		return Kap_Main::IppSelector;
		
	default:
		break;
	};
	return Kap_Main::Invalid;
}

void Kap_LoginSelector::guestClicked()
{
	bool	val = guest_->isChecked();
	login_->setEnabled(!val);
	passwd_->setEnabled(!val);
}
