#include "kap_printeroption.h"
#include "kups_printerconfigurator.h"
#include <qcombobox.h>
#include <qlabel.h>
#include <qlistbox.h>

Kap_PrinterOption::Kap_PrinterOption(ppd_option_t *option, QWidget *parent)
	: QWidget(parent)
{
	optionname_ = 0;
	optionwidget_ = 0;
	keywords_.setAutoDelete(true);
	if (option) init(option);
}

Kap_PrinterOption::~Kap_PrinterOption()
{
}

void Kap_PrinterOption::init(ppd_option_t *option)
{
	if (option == 0) return;
	option_ = option;
	optionname_ = new QLabel(this);
	optionname_->setAlignment(AlignVCenter | AlignRight);
	optionname_->setText(option->text);
	initWidget();
}

void Kap_PrinterOption::initWidget()
{
	switch (option_->ui)
	{
	   case PPD_UI_BOOLEAN:
	   case PPD_UI_PICKONE:
	   	optionwidget_ = new QComboBox(this);
	   	connect(optionwidget_,SIGNAL(activated(int)),SLOT(slotOptionSelected(int)));
	   	for (int i=0;i<option_->num_choices;i++)
	   	{
	   		ppd_choice_t	*choice = option_->choices+i;
	   		keywords_.append(choice->choice);
	   		((QComboBox*)optionwidget_)->insertItem(choice->text);
	   		if (choice->marked) ((QComboBox*)optionwidget_)->setCurrentItem(keywords_.count()-1);
	   	}
	   	break;
	   case PPD_UI_PICKMANY:
	   	optionwidget_ = new QListBox(this);
	   	((QListBox*)optionwidget_)->setMultiSelection(true);
	   	((QListBox*)optionwidget_)->setFixedVisibleLines(3);
	   	connect(optionwidget_,SIGNAL(highlighted(int)),SLOT(slotOptionSelected(int)));
	   	for (int i=0;i<option_->num_choices;i++)
	   	{
	   		ppd_choice_t	*choice = option_->choices+i;
	   		keywords_.append(choice->choice);
	   		((QListBox*)optionwidget_)->insertItem(choice->text);
	   		if (choice->marked) ((QListBox*)optionwidget_)->setSelected(keywords_.count()-1,true);
	   	}
	   	break;
	}
}

void Kap_PrinterOption::resizeEvent(QResizeEvent *)
{
	if (!optionwidget_ || !optionname_) return;
	optionname_->setGeometry(0,0,width()/2-5,height());
	QSize	s(optionwidget_->sizeHint());
	optionwidget_->setGeometry(width()/2+5,0,width()/2-5,height());
}

void Kap_PrinterOption::slotOptionSelected(int index)
{
	ppdMarkOption(Kups_PrinterConfigurator::getPPDFile()->ppd,option_->keyword,keywords_.at(index));
}

QSize Kap_PrinterOption::sizeHint()
{
	if (!optionname_ || !optionwidget_) return QSize(-1,-1);
	QSize	s1(optionname_->sizeHint()),s2(optionwidget_->sizeHint());
	return QSize(QMAX(s1.width(),s2.width())*2+10,QMAX(s1.height(),s2.height()));
}
