#include "kinstancemanager.h"

#include <qfile.h>
#include <qtextstream.h>
#include <qfileinfo.h>
#include <klocale.h>
#include <kapp.h>

#include <stdlib.h>
#include <qtcups/qcupsprinterconfigdialog.h>

#include "kups_infos.h"
#include "kprintermanager.h"
#include "kprinterinstance.h"
#include "kups_testpagegenerator.h"
#include "kbuffer.h"
#include "kprintermanager.h"
#include "kprinter.h"

KInstanceManager* KInstanceManager::unique_ = 0;

KInstanceManager::KInstanceManager()
{
	instances_.setAutoDelete(false);
	lastcheck_.setTime_t(0u);
	defaultprinter_ = "";
}

KInstanceManager::~KInstanceManager()
{
}

KInstanceManager* KInstanceManager::get()
{
	if (!unique_)
	{
		unique_ = new KInstanceManager();
	}
	return unique_;
}

void KInstanceManager::release()
{
	if (unique_)
	{
		delete unique_;
		unique_ = 0;
	}
}

//-------------------------------------------------------------------------
bool KInstanceManager::refresh()
{
	QFileInfo	fglobal, flocal;

	fglobal.setFile((Kups_Infos::get()->isStandardCUPS() ? QString("") : Kups_Infos::get()->getCUPSRoot()) + "/etc/cups/lpoptions");
	flocal.setFile(getenv("HOME") + QString("/.lpoptions"));
	if (QMAX(fglobal.lastModified(),flocal.lastModified()) > lastcheck_)
	{
		lastcheck_ = QMAX(flocal.lastModified(), fglobal.lastModified());
		for (instances_.first();instances_.current();instances_.next())
		{
			instances_.current()->setIsDiscarded(true);
			instances_.current()->cleanOptions();
		}
		loadInstanceFile(fglobal.absFilePath());
		loadInstanceFile(flocal.absFilePath());
		for (int i=0;i<(int)(instances_.count());i++)
		{
			if (instances_.at(i)->isDiscarded())
			{
				instances_.remove(i);
				i--;
			}
		}
	}
	return true;
}
/*
bool KInstanceManager::refresh()
{
	QFileInfo	fglobal, flocal;
	int	n(0);
	cups_dest_t	*d(0);

	fglobal.setFile((Kups_Infos::get()->isStandardCUPS() ? QString("") : Kups_Infos::get()->getCUPSRoot()) + "/etc/cups/lpoptions");
	flocal.setFile(getenv("HOME") + QString("/.lpoptions"));
	if (QMAX(fglobal.lastModified(),flocal.lastModified()) > lastcheck_)
	{
		lastcheck_ = QMAX(fglobal.lastModified(), flocal.lastModified());
		if (!loadInstanceFile(fglobal.absFilePath(), n, &d))
		{
			KPrinterManager::get()->setLastError(i18n("Unable to load %s").arg(fglobal.absFilePath()));
			return false;
		}
		if (!loadInstanceFile(flocal.absFilePath(), n, &d))
		{
			KPrinterManager::get()->setLastError(i18n("Unable to load %s").arg(flocal.absFilePath()));
			return false;
		}
	}
	if (n > 0)
	{
		for (instances_.first();instances_.current();instances_.next())
			instances_.current()->setIsDiscarded(true);
		for (int i=0;i<n;i++)
		{
			KPrinterInstance	*aInstance = new KPrinterInstance();
			aInstance->setCupsDest(&(d[i]));
			KPrinterInstance	*inst = findInstance(aInstance->getPrinter(), aInstance->getInstance());
			if (inst)
			{
				inst->copy(*aInstance);
				inst->setIsDiscarded(false);
				delete aInstance;
				aInstance = 0;
			}
			else
			{
				instances_.append(aInstance);
			}
		}
		for (int i=0;i<(int)(instances_.count());i++)
		{
			if (instances_.at(i)->isDiscarded())
			{
				instances_.remove(i);
				i--;
			}
		}
		cupsFreeDests(n, d);
	}

	return true;
}
*/
KPrinterInstance* KInstanceManager::findInstance(const QString& printer, const QString& instance)
{
	for (instances_.first();instances_.current();instances_.next())
		if (instances_.current()->getPrinter() == printer &&
		    instances_.current()->getInstance() == instance)
			return instances_.current();
	return 0;
}

QList<KPrinterInstance> KInstanceManager::getInstancesForPrinter(const QString& printer)
{
	QList<KPrinterInstance>	list;
	list.setAutoDelete(false);
	for (instances_.first();instances_.current();instances_.next())
		if (instances_.current()->getPrinter() == printer)
			list.append(instances_.current());
	return list;
}

bool KInstanceManager::saveInstances()
{
	QString	filename;
	if (getuid() == 0)
		filename = (Kups_Infos::get()->isStandardCUPS() ? QString("") : Kups_Infos::get()->getCUPSRoot()) + "/etc/cups/lpoptions";
	else
		filename = QString(getenv("HOME")) + "/.lpoptions";
	if (!writeInstanceFile(filename))
		return false;
	QFileInfo	fi(filename);
	lastcheck_ = fi.lastModified();
	return true;
}

//-------------------------------------------------------------------------
/*
bool KInstanceManager::loadInstanceFile(const QString& filename, int& num_dests, cups_dest_t **dests)
{
	QFile	f(filename);
	if (!f.exists()) return true;	// this doesn't mean an error
        if (!f.open(IO_ReadOnly)) return false;
	QTextStream	t(&f);
	QString	line, printer, instance;
        QStringList	list;
        while (!t.eof())
        {
        	line = t.readLine().simplifyWhiteSpace();
                list = QStringList::split(' ', line, false);
                if (list.count() < 2) continue;
                int	p = list[1].find('/');
                if (p != -1)
                {
                	printer = list[1].left(p);
                        instance = list[1].right(list[1].length()-p-1);
                }
                else
                {
                	printer = list[1];
                        instance = "\0";
                }
                num_dests = cupsAddDest(printer.data(), instance.data(), num_dests, dests);
		cups_dest_t	*d = cupsGetDest(printer.data(), instance.data(), num_dests, *dests);
                if (!d) return false;
                if (d->num_options > 0)
                {
                	cupsFreeOptions(d->num_options, d->options);
                	d->num_options = 0;
                	d->options = NULL;
		}
                d->num_options = cupsParseOptions(line.data()+(list[0].length()+list[1].length()+1), d->num_options, &(d->options));
        	if (list[0].lower() == "default")
                {
	                for (int i=0;i<num_dests;i++)
	                	(*dests)[i].is_default = 0;
			d->is_default = 1;
		}
        }
        return true;
}
*/
bool KInstanceManager::loadInstanceFile(const QString& filename)
{
	QFile	f(filename);
	if (!f.exists() || !f.open(IO_ReadOnly)) return true;
	QTextStream	t(&f);
	KPrinterInstance	*aInstance;
	QString	line;
	while (!t.eof())
	{
		line = t.readLine().stripWhiteSpace();
		if (line.isEmpty()) continue;	// skip blank lines
		aInstance = new KPrinterInstance;
		if (aInstance->loadFromString(line))
		{
			if (aInstance->isDefault())
			{
				clearDefaultInstance();
				defaultprinter_ = aInstance->getPrinter();
			}
			KPrinterInstance	*inst = findInstance(aInstance->getPrinter(), aInstance->getInstance());
			if (inst)
			{
				inst->copy(*aInstance);
				inst->setIsDiscarded(false);
				delete aInstance;
				aInstance = 0;
			}
			else
			{
				instances_.append(aInstance);
			}
		}
		else
		{
			delete aInstance;
			aInstance = 0;
		}
	}
	return true;
}

bool KInstanceManager::writeInstanceFile(const QString& filename)
{
	QFile	f(filename);
	if (!f.open(IO_WriteOnly))
	{
		errormsg_ = i18n("Unable to save instance file \"%1\"").arg(filename);
		return false;
	}
	QString	line;
	QTextStream	t(&f);
	for (instances_.first();instances_.current();instances_.next())
	{
		instances_.current()->writeToString(line);
		t << line << endl;
	}
	return true;
}

void KInstanceManager::clearDefaultInstance()
{
	for (instances_.first();instances_.current();instances_.next())
		instances_.current()->setDefault(false);
	defaultprinter_ = "";
}

//-------------------------------------------------------------------------
bool KInstanceManager::createInstance(const QString& pr, const QString& inst)
{
	if (findInstance(pr, inst))
	{
		errormsg_ = i18n("The instance \"%1\" already exists !").arg(inst);
		return false;
	}
	instances_.append(new KPrinterInstance(pr, inst));
	return saveInstances();
}

bool KInstanceManager::copyInstance(const QString& pr, const QString& inst, const QString& newinst)
{
	KPrinterInstance	*pinst = findInstance(pr, inst), *newpinst;
	if (!pinst)
	{
		errormsg_ = i18n("Unable to find instance \"%1\" !").arg(inst);
		return false;
	}
	newpinst = new KPrinterInstance;
	newpinst->copy(*pinst);
	newpinst->setInstance(newinst);
	newpinst->setDefault(false);
	instances_.append(newpinst);
	return saveInstances();
}

bool KInstanceManager::removeInstance(const QString& pr, const QString& inst)
{
	KPrinterInstance	*pinst = findInstance(pr, inst);
	if (!pinst)
	{
		errormsg_ = i18n("Unable to find instance \"%1\" !").arg(inst);
		return false;
	}
	if (pinst->isDefault())
	{
		defaultprinter_ = "";
		KPrinterManager::get()->refresh();
	}
	instances_.removeRef(pinst);
	return saveInstances();
}

bool KInstanceManager::setDefaultInstance(const QString& pr, const QString& inst)
{
	KPrinterInstance	*pinst = findInstance(pr, inst);
	if (!pinst)
	{ // instance doesn't exists, create it
		if (!createInstance(pr, inst))
		{
			errormsg_ = i18n("Unable to create instance \"%1\" !").arg(inst);
			return false;
		}
		pinst = findInstance(pr, inst);
		if (!pinst)
		{ // instance created, but not found !!!
			errormsg_ = i18n("Unable to find instance \"%1\" !").arg(inst);
			return false;
		}
	}
	clearDefaultInstance();
	pinst->setDefault(true);
	defaultprinter_ = pr;
	return saveInstances();
}

bool KInstanceManager::editInstance(const QString& pr, const QString& inst)
{
	KPrinterInstance	*pinst = findInstance(pr, inst);
	if (!pinst)
	{
		errormsg_ = i18n("Unable to find instance \"%1\" !").arg(inst);
		return false;
	}
	cups_dest_t	*d = pinst->getCupsDest();
	QCupsPrinterConfigDialog	dlg(0xFF, kapp->mainWidget(), "dialog", false);
	dlg.initialize(d);
	if (dlg.exec())
	{
		cupsFreeOptions(d->num_options, d->options);
		d->num_options = 0;
		d->options = NULL;
		dlg.getCupsOptions(d->num_options, &(d->options));
		pinst->setCupsDest(d);
		cupsFreeDests(1, d);
		return saveInstances();
	}
	else return true;
}

bool KInstanceManager::testInstance(const QString& pr, const QString& inst)
{
	KPrinterInstance	*pinst = findInstance(pr, inst);
	if (!pinst)
	{
		errormsg_ = i18n("Unable to find instance \"%1\" !").arg(inst);
		return false;
	}
	cups_dest_t	*d = pinst->getCupsDest();
	KBuffer	buf = Kups_TestPageGenerator::getPage(KPrinterManager::get()->getPrinterByName(pr));
	QString	tmpfile = cupsTempFile(NULL, 0);
	bool retVal(true);
	if (!buf.dump(tmpfile))
	{
		errormsg_ = i18n("Unable to create temporary file \"%1\" !").arg(tmpfile);
		retVal = false;
	}
	else if (cupsPrintFile(pr.data(), tmpfile.data(), "KUPS Instance Test", d->num_options, d->options) <= 0)
	{
		errormsg_ = i18n("Unable to send print job to printer \"%1\" !\n").arg(pr);
		errormsg_.append(ippErrorString(cupsLastError()));
		retVal = false;
	}
	cupsFreeDests(1, d);
	unlink(tmpfile.data());
	return retVal;
}

QString KInstanceManager::getPrinterDefaultInstance()
{
	return defaultprinter_;
}
