/***************************************************************************
                          kjobhelper.cpp  -  description
                             -------------------
    begin                : Wed Dec 20 2000
    copyright            : (C) 2000 by Michael Goffioul
    email                : goffioul@imec.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qmessagebox.h>
#include <klocale.h>
#include <kapp.h>

#include "kjobhelper.h"
#include "kprintermanager.h"
#include "kjob.h"
#include "kups_ipprequest.h"
#include "kups_infos.h"



// static
KJobHelper* KJobHelper::unique_ = 0;


KJobHelper::KJobHelper()
{
}
KJobHelper::~KJobHelper()
{
}

KJobHelper* KJobHelper::get()
{
	if (unique_ == 0)
	{
		unique_ = new KJobHelper;
	}
	return unique_;
}

void KJobHelper::setAttribute(KJob* job, ipp_attribute_t* attr)
{
#ifdef DEBUG
//debug("%s = %s",attr->name,attr->value_tag>=IPP_TAG_TEXT?attr->values[0].string.text:"not printable");
#endif
	if ( strcmp(attr->name, "printer-name") == 0)
	{
		QMessageBox::warning(kapp->mainWidget(),i18n("Jobs administrator"),i18n("Attribute with tag 'printer-name' !"),QMessageBox::Ok|QMessageBox::Default,0);

		KPrinterManager* mgr = KPrinterManager::get();
		KPrinter* p = mgr->getPrinterByName( attr->values[0].string.text );
		if (p != 0)
		{
			job->setPrinter(p);
		}
		else
		{
			QMessageBox::critical(kapp->mainWidget(),i18n("Jobs administrator"),i18n("Can't find specified printer !"),QMessageBox::Ok|QMessageBox::Default,0);
		}
	}
	else if ( strcmp(attr->name, "job-printer-uri") == 0)
	{
		KPrinterManager* mgr = KPrinterManager::get();
		KPrinter* p = mgr->getPrinterByUri( attr->values[0].string.text );
		if (!p) p = mgr->getClassByUri( attr->values[0].string.text );
		if (p != 0)
		{
			job->setPrinter(p);
		}
		else
		{
//			QMessageBox::critical(kapp->mainWidget(),i18n("Jobs administrator"),i18n("Can't find specified printer !"),QMessageBox::Ok|QMessageBox::Default,0);
			debug("Can't find printer for job %d",job->getID());
			job->setPrinter(0);
                }
	}
	else if ( strcmp(attr->name, "job-id") == 0)
	{
		job->setID(attr->values[0].integer);
	}
	else if ( strcmp(attr->name, "job-name") == 0)
	{
		job->setName(attr->values[0].string.text);
	}
	else if ( strcmp(attr->name, "job-k-octets") == 0)
	{
		job->setSize(attr->values[0].integer);
	}
	else if ( strcmp(attr->name, "job-more-info") == 0)
	{
	}
	else if ( strcmp(attr->name, "job-originating-user-name") == 0)
	{
		job->setUserName(attr->values[0].string.text);
	}
	else if ( strcmp(attr->name, "job-state") == 0)
	{
		job->setState( (ipp_jstate_t)attr->values[0].integer );
	}
	else if ( strcmp(attr->name, "job-uri") == 0)
	{
		job->setUri(attr->values[0].string.text);
	}
	else
	{
		sleep(0);
	}
}

bool KJobHelper::sendJobCommand(KJob *job, KJob::Command command)
{
	switch (command)
	{
	   case KJob::Remove:
	   case KJob::RemoveAll:
	   	return false;
	   case KJob::Hold:
	   case KJob::Restart:
	   case KJob::Release:
	   	return doJobCommand(job,command);
		 default:
			break;
	}
	KPrinterManager::setLastError("Not implemented yet !");
	return false;
}

bool KJobHelper::sendJobCommand(KJob *job, KJob::Command command, KPrinter *printer)
{
	switch (command)
	{
	   case KJob::Move:
	   	return moveJob(job,printer);
		 default:
			break;
	}
	KPrinterManager::setLastError("Not implemented yet !");
	return false;
}

bool KJobHelper::doJobCommand(KJob *job, KJob::Command command)
{
	Kups_IppRequest	request;

	request.setUri("job-uri", job->getUri());
	request.setName("requesting-user-name", Kups_Infos::get()->getUser().data() );
	switch (command)
	{
	   case KJob::Hold:
	   	request.setOperation(IPP_HOLD_JOB);
	   	break;
	   case KJob::Release:
	   	request.setOperation(IPP_RELEASE_JOB);
	   	break;
	   case KJob::Restart:
	   	request.setOperation(IPP_RESTART_JOB);
	   	break;
	   default:
			return false;
	}
	if (!request.doRequest("/jobs/"))
	{
		return false;
	}
	else if (request.getResponseStatus() == IPP_NOT_FOUND)
	{
		KPrinterManager::setLastError(i18n("Job %1 (%2) does not exist !").arg(job->getName()).arg(job->getID()));
		return false;
	}

	return true;
}

bool KJobHelper::moveJob(KJob *job, KPrinter *printer)
{
	Kups_IppRequest	request;
	QString	juri = job->getUri();
	QString	puri = printer->getIppURI();

	request.setOperation(CUPS_MOVE_JOB);
	request.setUri("job-uri",job->getUri());
	request.setName("requesting-user-name",Kups_Infos::get()->getUser());
	request.setUri("job-printer-uri",printer->getIppURI());
	//request.setUri(IPP_TAG_JOB,"job-printer-uri",puri);

	if (!request.doRequest("/jobs/"))
	{
		return false;
	}
	else if (request.getResponseStatus() != IPP_OK)
	{
		QString	msg;
		Kups_Helper::getErrorString(&request,printer,msg);
		KPrinterManager::setLastError(i18n("Unable to move job.\nError: %1").arg(msg));
		return false;
	}

	return true;
}
