#ifndef KPRINTERINSTANCE_H
#define KPRINTERINSTANCE_H

#include <qdict.h>
#include <qstring.h>

#include <cups/cups.h>

#include "kobject.h"

struct KInstanceOption
{
	KInstanceOption(const char *opt, const char *val) : option_(opt), value_(val) {}
	KInstanceOption(const QString& opt, const QString& val) : option_(opt), value_(val) {}
	KInstanceOption(const KInstanceOption& o) : option_(o.option_), value_(o.value_) {}
	QString	option_;
	QString value_;
};

class KPrinterInstance : public KObject
{
public:
	KPrinterInstance();
	KPrinterInstance(const QString& printer, const QString& instance);
	~KPrinterInstance();
	void copy(const KPrinterInstance& p);

	void setCupsDest(cups_dest_t *d);
	cups_dest_t* getCupsDest();
	const QString& getPrinter();
	void setPrinter(const QString& pr) { printer_ = pr; }
	const QString& getInstance() const;
	void setInstance(const QString& inst) { instance_ = inst; }
	bool isDefault() const { return isDefault_; }
	void setDefault(bool on) { isDefault_ = on; }

	bool loadFromString(const QString& line);
	void writeToString(QString& line);
	void cleanOptions() { options_.clear(); }

private:
	QString	printer_;
	QString	instance_;
	QDict<KInstanceOption>	options_;
	bool	isDefault_;
};

#endif
