/***************************************************************************
                          kups.h  -  description
                             -------------------
    begin                : Mon Nov  8 19:59:21 CET 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KUPS_H
#define KUPS_H

#include <qglobal.h>

#include <kapp.h>
#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>

#include <cups/cups.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


// include files for KDE 
#include <ktmainwindow.h>
#include <kaccel.h>

// forward declaration of the Kups classes
class KupsView;

/**
  * The base class for Kups application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KupsView creates your center view, which is connected  * to the window's Doc object.
  * KupsApp reimplements the methods that KTMainWindow provides for main window handling and supports
  * full session management as well as keyboard accelerator configuration by using KAccel.
  * @see KTMainWindow
  * @see KApplication
  * @see KConfig
  * @see KAccel
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 0.4 code generation
  */
class KupsApp : public KTMainWindow
{
  Q_OBJECT

  friend class KupsView;

	private:	
		static KupsApp* unique_;
		
  public:
	  static KupsApp* get();
	
    /** construtor of KupsApp, calls all init functions to create the application.
     * @see initMenuBar initToolBar
     */
    KupsApp();
    ~KupsApp();
    /** enables menuentries/toolbar items
     */
//    void enableCommand(int id_);
    /** disables menuentries/toolbar items
     */
//    void disableCommand(int id_);

    bool isValid() { return isValid_; };

  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initKeyAccel creates the keyboard accelerator items for the available slots and changes the menu accelerators.
     * @see KAccel
     */
//    void initKeyAccel();
    /** initMenuBar creates the menubar and inserts the menupopups as well as creating the helpMenu.
     * @see KApplication#getHelpMenu
     */
//    void initMenuBar();
    /** this creates the toolbars.
     */
//    void initToolBar();
    /** sets up the toolbar and menubar.
     */
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    bool initView();
    /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** saves the window properties for each open window during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    virtual void saveProperties(KConfig *_cfg);
    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    virtual void readProperties(KConfig *_cfg);

    void checkCUPSAccess();

  public slots:
    /** switch argument for slot selection by menu or toolbar ID */
//    void commandCallback(int id_);
    /** switch argument for Statusbar help entries on slot selection. Add your ID's help here for toolbars and menubar entries. */
    void statusCallback(int id_);

    void slotFileOptions();

    void slotPrintingTest();
    void slotPrintingPSFile();
    /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();

    void slotPrinterAdd();
    void slotConfigureToolbars();
    void slotRefresh();

    /** toggles the toolbar
     */
    void slotViewToolBar();
    /** toggles the statusbar
     */
    void slotViewStatusBar();

    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);
    /** changes the status message of the whole statusbar for two seconds, then restores the last status. This is used to display
     * statusbar messages that give information about actions for toolbar icons and menuentries.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusHelpMsg(const QString &text);

    void slotConfigureServer();
		void slotRestartServer();

  private:

    /** the configuration object of the application */
    KConfig *config_;
    /** the key accelerator container */
//    KAccel *keyAccel_;
    /** file_menu contains all items of the menubar entry "File" */
//    QPopupMenu* fileMenu_;
//    QPopupMenu* printingMenu_;
//    QPopupMenu* printerMenu_;
    /** view_menu contains all items of the menubar entry "View" */
//    QPopupMenu *viewMenu_;
    /** help_menu contains all items of the menubar entry "Help" */
//    QPopupMenu *helpMenu_;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    KupsView *view_;

    bool isValid_;
};
 
#endif // KUPS_H
