

#include <cups/cups.h>

#include <qstring.h>

#include <klocale.h>

#include <ctype.h>

#include "kups.h"
#include "kups_printerconfigurator.h"
#include "kprinter.h"
#include "kap_printerconfig.h"
#include "kups_ipprequest.h"
#include "kprintermanager.h"
#include "kups_infos.h"



global_ppd_file_t*	Kups_PrinterConfigurator::ppdfile_ = 0;

bool writePPDFile(const char *src, const char *dest, ppd_file_t *ppdfile);

Kups_PrinterConfigurator::Kups_PrinterConfigurator()
{
}

Kups_PrinterConfigurator::~Kups_PrinterConfigurator()
{
}

bool Kups_PrinterConfigurator::configurePrinter(KPrinter *printer, QWidget *parent)
{
	QString	ppdname = cupsGetPPD(printer->getName().data());
	ppdfile_ = 0;
	bool	retVal(true);
	if (!ppdname.isEmpty()) ppdfile_ = globalPpdOpenFile(ppdname.data());
	if (!ppdfile_)
	{
		KPrinterManager::setLastError(i18n("Unable to load driver file for \"%1\"").arg(printer->getName()));
		return false;
	}
	globalPpdMarkDefaults(ppdfile_);
	Kap_PrinterConfig	*Dlg = new Kap_PrinterConfig(ppdfile_,parent);
	Dlg->setCaption(printer->getDescription().data());
	if (Dlg->exec())
	{
		// first recopy current PPD file with new settings
		char	tmpfile[1024];
		cupsTempFile(tmpfile,1024);
		if (writeGlobalPpdFile(ppdfile_,tmpfile,ppdname.data()) == -1)
		{
			retVal = false;
			KPrinterManager::setLastError("Internal error, unable to write PPD file");
		}
		else
		{
			// then send request to modify printer configuration
			Kups_IppRequest	request;
			request.setOperation(CUPS_ADD_PRINTER);
			QString	uri = printer->getIppURI();
			request.setUri("printer-uri",uri);
			QString	res("/admin/");
			if (!request.doFileRequest(res,tmpfile)) retVal = false;
			else
			{
				ipp_status_t status = request.getResponseStatus();
				if (status == IPP_NOT_FOUND)
				{
					QString error;
					error.sprintf(i18n("Destination %s does not exist!\n"), printer->getName().data());
					KPrinterManager::setLastError(error);
					retVal = false;
				}
				else if (status != IPP_OK)
				{
					QString error;
					error.sprintf(i18n("Can't configure printer : returned error code %d"), status);
					KPrinterManager::setLastError(error);
					retVal = false;
				}
			}
		}
		unlink(tmpfile);
	}
	else
	{
		retVal = false;
		KPrinterManager::setLastError("");
	}
	delete Dlg;
	globalPpdClose(ppdfile_);
	unlink(ppdname.data());
	ppdfile_ = 0;
	return retVal;
}

global_ppd_file_t* Kups_PrinterConfigurator::getPPDFile()
{
	return ppdfile_;
}

//-----------------------------

#include <stdio.h>

bool writePPDFile(const char *src, const char *dest, ppd_file_t *ppdfile)
{
	FILE	*in, *out;
	bool	retVal(true);

	in = fopen(src,"r");
	out = fopen(dest,"w");
	if (in && out)
	{
		char	linebuf[1024];
		char	keyword[64] = {0};
		while (fgets(linebuf,1024,in) != NULL)
		{
			if (strncmp(linebuf,"*Default",8) != 0)
				fprintf(out,"%s",linebuf);
			else
			{
				char	*c;
				strncpy(keyword,linebuf+8,63);
				for (c = keyword;*c;c++)
					if (*c == ':' || isspace(*c)) break;
				*c = '\0';
				ppd_choice_t	*choice = ppdFindMarkedChoice(ppdfile,(strcmp(keyword,"PageRegion") == 0 ? "PageSize" : keyword));
				if (choice)
					fprintf(out,"*Default%s: %s\n",keyword,choice->choice);
				else
					fprintf(out,"%s",linebuf);
			}
		}
	}
	else retVal = false;

	fclose(in);
	fclose(out);
	
	return retVal;
}
