#include "kups_printerinfo.h"

#include <qlabel.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qframe.h>
#include <klocale.h>

#include <cups/cups.h>

#include "kprinter.h"
#include "kups_printeritem.h"
#include "kups_printerproperties.h"

extern void stopTimer();
extern void startTimer();

Kups_PrinterInfo::Kups_PrinterInfo(QWidget *parent, const char *name)
	: QWidget(parent,name)
{
	currentPrinter_ = 0;
	item_ = 0;
	initWidget();
	updateWidget();
}

Kups_PrinterInfo::~Kups_PrinterInfo()
{
}

void Kups_PrinterInfo::initWidget()
{
	QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10, 0);

	nameLabel_ = new QLabel(this);
	QFont	f = nameLabel_->font();
	f.setWeight(QFont::Bold);
	nameLabel_->setFont(f);
	typeLabel_ = new QLabel(this);
	ippUriLabel_ = new QLabel(this);
	descrLabel_ = new QLabel(this);
	locLabel_ = new QLabel(this);
	stateLabel_ = new QLabel(this);
	makeAndModelLabel_ = new QLabel(this);
	deviceUriLabel_ = new QLabel(this);
	stateIconLabel_ = new QLabel(this);
	bannersLabel_ = new QLabel(this);

	QLabel	*l1 = new QLabel(i18n("Type:"), this);
	l1->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	QLabel	*l2 = new QLabel(i18n("URI:"), this);
	l2->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	QLabel	*l3 = new QLabel(i18n("Description:"), this);
	l3->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	QLabel	*l4 = new QLabel(i18n("Location:"), this);
	l4->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	QLabel	*l5 = new QLabel(i18n("State:"), this);
	l5->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	devMembersLabel_ = new QLabel(i18n("Device:"), this);
	devMembersLabel_->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	QLabel	*l6 = new QLabel(i18n("Model:"), this);
	l6->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	QLabel	*l7 = new QLabel(i18n("Banners:"), this);
	l7->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

	QFrame	*fr1 = new QFrame(this);
	fr1->setFrameStyle(QFrame::HLine|QFrame::Sunken);
	fr1->setLineWidth(1);
	fr1->setFixedHeight(10);

	QHBoxLayout	*nameLayout_ = new QHBoxLayout(mainLayout_);
	nameLayout_->addWidget(nameLabel_);
	stateIconLabel_->setFixedSize(QSize(16,16));
	nameLayout_->addWidget(stateIconLabel_);

	mainLayout_->addWidget(fr1);

#define	WIDGET_SPACING	3
	QHBoxLayout	*subLayout_ = new QHBoxLayout(0, 0, 5);
	QVBoxLayout	*leftLayout_ = new QVBoxLayout(0, 0, WIDGET_SPACING);
	QVBoxLayout	*rightLayout_ = new QVBoxLayout(0, 0, WIDGET_SPACING);
	mainLayout_->addLayout(subLayout_);
	subLayout_->addLayout(leftLayout_);
	subLayout_->addLayout(rightLayout_, 1);

	leftLayout_->addWidget(l1);
	leftLayout_->addWidget(l2);
	leftLayout_->addWidget(l3);
	leftLayout_->addWidget(l4);
	leftLayout_->addWidget(l5);
	leftLayout_->addSpacing(10);
	leftLayout_->addWidget(devMembersLabel_);
	leftLayout_->addWidget(l6);
	leftLayout_->addSpacing(10);
	leftLayout_->addWidget(l7);

	rightLayout_->addWidget(typeLabel_);
	rightLayout_->addWidget(ippUriLabel_);
	rightLayout_->addWidget(descrLabel_);
	rightLayout_->addWidget(locLabel_);
	rightLayout_->addWidget(stateLabel_);
	rightLayout_->addSpacing(10);
	rightLayout_->addWidget(deviceUriLabel_);
	rightLayout_->addWidget(makeAndModelLabel_);
	rightLayout_->addSpacing(10);
	rightLayout_->addWidget(bannersLabel_);

	mainLayout_->addStretch(1);
}

void Kups_PrinterInfo::updateWidget()
{
	if (!currentPrinter_ || currentPrinter_->isDiscarded())
	{
		nameLabel_->setText("");
		typeLabel_->setText("");
		descrLabel_->setText("");
		locLabel_->setText("");
		stateLabel_->setText("");
		ippUriLabel_->setText("");
		makeAndModelLabel_->setText("");
		deviceUriLabel_->setText("");
		stateIconLabel_->setPixmap(QPixmap());
		bannersLabel_->setText("");
	}
	else
	{
		nameLabel_->setText(currentPrinter_->getName());
		descrLabel_->setText(currentPrinter_->getDescription());
		locLabel_->setText(currentPrinter_->getLocation());
		ippUriLabel_->setText(currentPrinter_->getIppURI());
		if (currentPrinter_->isImplicit())
			makeAndModelLabel_->setText("");
		else
			makeAndModelLabel_->setText(currentPrinter_->getModelInfo());
		if (currentPrinter_->getPrinterType() & CUPS_PRINTER_CLASS)
		{
			devMembersLabel_->setText(i18n("Members:"));
			bannersLabel_->setText(i18n("Undefined"));
			KClassPrinter	*classe = (KClassPrinter*)currentPrinter_;
			int	n = classe->getNbPrinters();
			if (n == 0) deviceUriLabel_->setText("");
			else
			{
				QString	str;
				for (int i=0;i<n;i++)
				{
					KPrinter	*pr = classe->getPrinterAt(i);
					str += (pr->getName() + ", ");
				}
				deviceUriLabel_->setText(str.left(str.length()-2));
			}
		}
		else
		{
			deviceUriLabel_->setText(currentPrinter_->getLocalURI());
			devMembersLabel_->setText(i18n("Device:"));
			if (currentPrinter_->isRemote() || currentPrinter_->isImplicit())
				bannersLabel_->setText(i18n("Undefined"));
			else
			{
				QString	s = QString("%1, %2").arg(currentPrinter_->getStartBanner()).arg(currentPrinter_->getEndBanner());
				bannersLabel_->setText(s);
			}
		}
		if (item_)
		{
			stateIconLabel_->setPixmap(*(item_->pixmap(0)));
			stateIconLabel_->setFixedSize(stateIconLabel_->sizeHint());
		}
		if (currentPrinter_->isImplicit()) typeLabel_->setText(i18n("Implicit class"));
		else typeLabel_->setText(i18n("%1 %2").arg(currentPrinter_->isRemote() ? i18n("Remote") : i18n("Local")).arg(currentPrinter_->getPrinterType() & CUPS_PRINTER_CLASS ? i18n("class") : i18n("printer")));
		switch (currentPrinter_->getState())
		{
		   case KPrinter::Idle:
		   	stateLabel_->setText(i18n("Idle"));
		   	break;
		   case KPrinter::Processing:
		   	stateLabel_->setText(i18n("Processing"));
		   	break;
		   case KPrinter::Stopped:
		   	stateLabel_->setText(i18n("Stopped (%1)").arg(currentPrinter_->getStateReason().isEmpty() ? i18n("Unknown reason") : currentPrinter_->getStateReason()));
		   	break;
		}
	}
}

void Kups_PrinterInfo::printerSelected(QListViewItem *item)
{
	item_ = (Kups_PrinterItem*)item;
	if (!item_) currentPrinter_ = 0;
	else currentPrinter_ = item_->getPrinter();
	updateWidget();
}
