#include "cupslocationdialog.h"

#include <klocale.h>
#include <qmessagebox.h>
#include <qwhatsthis.h>

#include "cupslocationgeneral.h"
#include "cupslocationaccess.h"
#include "cupsdconf.h"

CupsLocationDialog::CupsLocationDialog(CupsdConf *conf, QWidget *parent, const char *name)
	: QTabDialog(parent, name, true)
{
	conf_ = conf;

	general_ = new CupsLocationGeneral(conf_, this);
	addTab(general_, i18n("General"));
        general_->setInfos(conf);

	access_ = new CupsLocationAccess(this);
	addTab(access_, i18n("Access"));
        access_->setInfos(conf);

	setOkButton(i18n("OK"));
	setCancelButton(i18n("Cancel"));

	setHelpButton(i18n("Short help..."));
        connect(this, SIGNAL(helpButtonPressed()), SLOT(slotHelp()));

	setCaption(i18n("Add resource"));
}

CupsLocationDialog::~CupsLocationDialog()
{
}

void CupsLocationDialog::loadLocation(CupsLocation *loc)
{
	general_->loadLocation(loc);
	access_->loadLocation(loc);

	setCaption(i18n("Resource \"%1\"").arg(loc->resourcename_));
}

void CupsLocationDialog::saveLocation(CupsLocation *loc)
{
	if (!general_->isValid())
		QMessageBox::critical(this, i18n("Error"), i18n("You must specify a resource name !"), QMessageBox::Ok|QMessageBox::Default, 0);
	else
	{
		general_->saveLocation(loc);
		access_->saveLocation(loc);
	}
}

void CupsLocationDialog::done(int result)
{
	if (result == Accepted)
	{
		if (!general_->isValid())
		{
			QMessageBox::critical(this, i18n("Error"), i18n("You must specify a resource name !"), QMessageBox::Ok|QMessageBox::Default, 0);
			return;
		}
	}
	QTabDialog::done(result);
}

void CupsLocationDialog::slotHelp()
{
	QWhatsThis::enterWhatsThisMode();
}
